"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const fs_1 = __importDefault(require("fs"));
const lodash_1 = __importDefault(require("lodash"));
const __1 = require("../");
beforeEach(() => {
    if (fs_1.default.existsSync('monika.har.yml')) {
        fs_1.default.unlinkSync('monika.har.yml');
    }
    if (fs_1.default.existsSync('monika.postman-basic.yml')) {
        fs_1.default.unlinkSync('monika.postman-basic.yml');
    }
    if (fs_1.default.existsSync('monika.postman-grouped.yml')) {
        fs_1.default.unlinkSync('monika.postman-grouped.yml');
    }
    if (fs_1.default.existsSync('sitemap.xml')) {
        fs_1.default.unlinkSync('sitemap.xml');
    }
    if (fs_1.default.existsSync('monika.insomnia.yml')) {
        fs_1.default.unlinkSync('monika.insomnia.yml');
    }
    if (fs_1.default.existsSync('monika.sitemap.yml')) {
        fs_1.default.unlinkSync('monika.sitemap.yml');
    }
});
describe('Har config', () => {
    describe('Create config from har file', () => {
        it('should create config from har file', async () => {
            const flags = {
                har: './src/components/config/__tests__/form_encoded.har',
                output: 'monika.har.yml',
            };
            await (0, __1.createConfig)(flags);
            (0, chai_1.expect)(fs_1.default.lstatSync('monika.har.yml').isFile()).to.be.true;
            const generated = fs_1.default.readFileSync('monika.har.yml', 'utf8');
            const expected = fs_1.default.readFileSync('./src/components/config/__tests__/expected.har.yml', 'utf8');
            (0, chai_1.expect)(lodash_1.default.isEqual(generated, expected)).to.be.true;
        });
    });
});
describe('Text config', () => {
    describe('Create config from text file', () => {
        it('should create config from text file', async () => {
            const flags = {
                text: './src/components/config/__tests__/textfile',
                output: 'monika.textfile.yml',
                force: true,
            };
            await (0, __1.createConfig)(flags);
            (0, chai_1.expect)(fs_1.default.lstatSync('monika.textfile.yml').isFile()).to.be.true;
            const generated = fs_1.default.readFileSync('monika.textfile.yml', 'utf8');
            const expected = fs_1.default.readFileSync('./src/components/config/__tests__/expected.textfile.yml', 'utf8');
            (0, chai_1.expect)(lodash_1.default.isEqual(generated, expected)).to.be.true;
        });
    });
});
describe('Sitemap config', () => {
    describe('Create config from xml file', () => {
        it('should create config from xml file', async () => {
            const flags = {
                sitemap: './src/components/config/__tests__/sitemap.xml',
                output: 'monika.sitemap.yml',
            };
            await (0, __1.createConfig)(flags);
            (0, chai_1.expect)(fs_1.default.lstatSync('monika.sitemap.yml').isFile()).to.be.true;
            const generated = fs_1.default.readFileSync('monika.sitemap.yml', 'utf8');
            const expected = fs_1.default.readFileSync('./src/components/config/__tests__/expected.sitemap.yml', 'utf8');
            (0, chai_1.expect)(lodash_1.default.isEqual(generated, expected)).to.be.true;
        });
    });
});
describe('Sitemap config [--one-probe ]', () => {
    describe('Create config from xml file', () => {
        it('should create config from xml file', async () => {
            const flags = {
                sitemap: './src/components/config/__tests__/sitemap.xml',
                output: 'monika.sitemap.yml',
                'one-probe': true,
            };
            await (0, __1.createConfig)(flags);
            (0, chai_1.expect)(fs_1.default.lstatSync('monika.sitemap.yml').isFile()).to.be.true;
            const generated = fs_1.default.readFileSync('monika.sitemap.yml', 'utf8');
            const expected = fs_1.default.readFileSync('./src/components/config/__tests__/expected.sitemap-oneprobe.yml', 'utf8');
            (0, chai_1.expect)(lodash_1.default.isEqual(generated, expected)).to.be.true;
        });
    });
});
const getPostmanConfig = ({ grouped }) => {
    if (grouped) {
        const generated = fs_1.default.readFileSync('monika.postman-grouped.yml', 'utf8');
        const expected = fs_1.default.readFileSync('./src/components/config/__tests__/mock_files/expected.postman-grouped.yml', 'utf8');
        return { generated, expected };
    }
    const generated = fs_1.default.readFileSync('monika.postman-basic.yml', 'utf8');
    const expected = fs_1.default.readFileSync('./src/components/config/__tests__/mock_files/expected.postman-basic.yml', 'utf8');
    return { generated, expected };
};
describe('Postman config', () => {
    describe('Create config from postman file', () => {
        it('[v2.0] - should create config from basic postman file', async () => {
            const flags = {
                postman: './src/components/config/__tests__/mock_files/basic-postman_collection-v2.0.json',
                output: 'monika.postman-basic.yml',
            };
            await (0, __1.createConfig)(flags);
            (0, chai_1.expect)(fs_1.default.lstatSync('monika.postman-basic.yml').isFile()).to.be.true;
            const { generated, expected } = getPostmanConfig({ grouped: false });
            (0, chai_1.expect)(lodash_1.default.isEqual(generated, expected)).to.be.true;
        });
        it('[v2.1] - should create config from basic postman file', async () => {
            const flags = {
                postman: './src/components/config/__tests__/mock_files/basic-postman_collection-v2.1.json',
                output: 'monika.postman-basic.yml',
            };
            await (0, __1.createConfig)(flags);
            (0, chai_1.expect)(fs_1.default.lstatSync('monika.postman-basic.yml').isFile()).to.be.true;
            const { generated, expected } = getPostmanConfig({ grouped: false });
            (0, chai_1.expect)(lodash_1.default.isEqual(generated, expected)).to.be.true;
        });
        it('[v2.0] - should create config from grouped postman file', async () => {
            const flags = {
                postman: './src/components/config/__tests__/mock_files/grouped-postman_collection-v2.0.json',
                output: 'monika.postman-grouped.yml',
            };
            await (0, __1.createConfig)(flags);
            (0, chai_1.expect)(fs_1.default.lstatSync('monika.postman-grouped.yml').isFile()).to.be.true;
            const { generated, expected } = getPostmanConfig({ grouped: true });
            (0, chai_1.expect)(lodash_1.default.isEqual(generated, expected)).to.be.true;
        });
        it('[v2.1] - should create config from grouped postman file', async () => {
            const flags = {
                postman: './src/components/config/__tests__/mock_files/grouped-postman_collection-v2.1.json',
                output: 'monika.postman-grouped.yml',
            };
            await (0, __1.createConfig)(flags);
            (0, chai_1.expect)(fs_1.default.lstatSync('monika.postman-grouped.yml').isFile()).to.be.true;
            const { generated, expected } = getPostmanConfig({ grouped: true });
            (0, chai_1.expect)(lodash_1.default.isEqual(generated, expected)).to.be.true;
        });
    });
});
describe('Insomnia config', () => {
    describe('Create config from insomnia file', () => {
        it('should create config from insomnia file', async () => {
            const flags = {
                insomnia: './src/components/config/__tests__/petstore.insomnia.yaml',
                output: 'monika.insomnia.yml',
            };
            await (0, __1.createConfig)(flags);
            (0, chai_1.expect)(fs_1.default.lstatSync('monika.insomnia.yml').isFile()).to.be.true;
            const generated = fs_1.default.readFileSync('monika.insomnia.yml', 'utf8');
            const expected = fs_1.default.readFileSync('./src/components/config/__tests__/expected.insomnia.yml', 'utf8');
            (0, chai_1.expect)(lodash_1.default.isEqual(generated, expected)).to.be.true;
        });
    });
});
//# sourceMappingURL=create-config.test.js.map