"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseConfig = void 0;
const parse_postman_1 = require("./parse-postman");
const parse_sitemap_1 = require("./parse-sitemap");
const parse_text_1 = require("./parse-text");
const parse_har_1 = require("./parse-har");
const path_1 = __importDefault(require("path"));
const js_yaml_1 = __importDefault(require("js-yaml"));
const parse_insomnia_1 = __importDefault(require("./parse-insomnia"));
const is_url_1 = __importDefault(require("is-url"));
const fetch_1 = require("./fetch");
const read_file_1 = require("../../utils/read-file");
function sleep(ms) {
    // eslint-disable-next-line no-promise-executor-return
    return new Promise((resolve) => setTimeout(resolve, ms));
}
const parseConfig = async (source, type, flag) => {
    try {
        let configString = (0, is_url_1.default)(source)
            ? await (0, fetch_1.fetchConfig)(source)
            : await (0, read_file_1.readFile)(source, 'utf8');
        if (configString.length === 0) {
            if ((0, is_url_1.default)(source))
                // was the remote file empty
                throw new Error(`The remote file ${source} is empty. Please check the URL or your connection again.`);
            let tries = 10; // tries multiple times to load the file
            while (configString.length === 0 && tries > 0) {
                sleep(700);
                // eslint-disable-next-line no-await-in-loop
                configString = await (0, read_file_1.readFile)(source, 'utf8');
                if (configString.length > 0) {
                    break;
                }
                tries--;
            }
            if (configString.length === 0)
                throw new Error(`Failed to read ${source}, got empty config string.`);
        }
        const ext = path_1.default.extname(source);
        if (type === 'har')
            return (0, parse_har_1.parseHarFile)(configString);
        if (type === 'text')
            return (0, parse_text_1.parseConfigFromText)(configString);
        if (type === 'postman')
            return (0, parse_postman_1.parseConfigFromPostman)(configString);
        if (type === 'sitemap')
            return (0, parse_sitemap_1.parseConfigFromSitemap)(configString, flag);
        if (type === 'insomnia')
            return (0, parse_insomnia_1.default)(configString, ext.replace('.', ''));
        if (ext === '.yml' || ext === '.yaml') {
            const cfg = js_yaml_1.default.load(configString, { json: true });
            return cfg;
        }
        return JSON.parse(configString);
    }
    catch (error) {
        const parsingError = error instanceof Error ? error : new Error(String(error));
        if (parsingError.name === 'SyntaxError') {
            throw new Error('JSON configuration file is in invalid JSON format!');
        }
        throw new Error(parsingError.message);
    }
};
exports.parseConfig = parseConfig;
//# sourceMappingURL=parse.js.map