"use strict";
/**********************************************************************************
 * MIT License                                                                    *
 *                                                                                *
 * Copyright (c) 2021 Hyperjump Technology                                        *
 *                                                                                *
 * Permission is hereby granted, free of charge, to any person obtaining a copy   *
 * of this software and associated documentation files (the "Software"), to deal  *
 * in the Software without restriction, including without limitation the rights   *
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell      *
 * copies of the Software, and to permit persons to whom the Software is          *
 * furnished to do so, subject to the following conditions:                       *
 *                                                                                *
 * The above copyright notice and this permission notice shall be included in all *
 * copies or substantial portions of the Software.                                *
 *                                                                                *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR     *
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,       *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE    *
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER         *
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,  *
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE  *
 * SOFTWARE.                                                                      *
 **********************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
exports.retryMaxDelayMs = exports.retryInitialDelayMs = exports.symonGetProbesIntervalMs = exports.symonAPIVersion = exports.monikaFlagsDefaultValue = exports.SYMON_API_VERSION = void 0;
const core_1 = require("@oclif/core");
const monika_flags_1 = require("./context/monika-flags");
var SYMON_API_VERSION;
(function (SYMON_API_VERSION) {
    SYMON_API_VERSION["v1"] = "v1";
    SYMON_API_VERSION["v2"] = "v2";
})(SYMON_API_VERSION || (exports.SYMON_API_VERSION = SYMON_API_VERSION = {}));
exports.monikaFlagsDefaultValue = {
    config: (0, monika_flags_1.getDefaultConfig)(),
    'config-filename': 'monika.yml',
    'config-interval': 900,
    'create-config': false,
    'native-fetch': false,
    flush: false,
    'follow-redirects': 21,
    force: false,
    'keep-verbose-logs': false,
    logs: false,
    'one-probe': false,
    repeat: 0,
    retryInitialDelayMs: 2000,
    retryMaxDelayMs: 30000,
    // default is 20s interval lookup
    stun: 20,
    summary: false,
    symonGetProbesIntervalMs: Number.parseInt(process.env.FETCH_PROBES_INTERVAL ?? '60000', 10),
    verbose: false,
    version: undefined,
};
exports.symonAPIVersion = core_1.Flags.custom({
    default: SYMON_API_VERSION.v1,
    description: 'Symon API version to use. Available options: v1, v2. Default: v1',
    options: [SYMON_API_VERSION.v1, SYMON_API_VERSION.v2],
});
exports.symonGetProbesIntervalMs = core_1.Flags.integer({
    default: exports.monikaFlagsDefaultValue.symonGetProbesIntervalMs,
    description: `To determine how often Monika sends a request to Symon to get probe data, in milliseconds. Defaults to ${exports.monikaFlagsDefaultValue.symonGetProbesIntervalMs}ms`,
});
exports.retryInitialDelayMs = core_1.Flags.integer({
    default: exports.monikaFlagsDefaultValue.retryInitialDelayMs,
    description: `The initial, first delay of the backoff retry when probe request is failed, in milliseconds. Defaults to ${exports.monikaFlagsDefaultValue.retryInitialDelayMs}ms`,
});
exports.retryMaxDelayMs = core_1.Flags.integer({
    default: exports.monikaFlagsDefaultValue.retryMaxDelayMs,
    description: `Maximum backoff retry delay, in milliseconds. Defaults to ${exports.monikaFlagsDefaultValue.retryMaxDelayMs}ms.`,
});
//# sourceMappingURL=flag.js.map